package fccsc.manager.data.process;

import java.util.*;
import javax.mail.MessagingException;

import intarsys.mail.MailSender;
import intarsys.util.*;
import intarsys.xml.SAXManager;
import intarsys.xml.handler.SingleTagHandler;

import fccsc.manager.broker.*;
import fccsc.manager.data.*;
import fccsc.manager.data.xml.*;


public final class Admission
	extends ProcessFXML
{
    public
	Admission()
    {
		super( "message.ADMISSION.properties" );
    }


	/**
	 * Interface Method - Processing method.
	 */
	public void
	process() throws Exception
	{
		super.process();

		// send the entire XML message
		String data = this.getXMLFactsMessage().getXML();

		EntireXBroker broker = new EntireXBroker( this.getProperties() );
		broker.sendMessage( data );

		// get response message
		String response   = broker.getResponse();
		String resRC      = ReturnCodes.CODE_00000;
		String resMessage = ReturnCodes.CODE_00000_MESSAGE;

		if ( logger.isDebugEnabled() )
		{
			logger.debug( "Broker Response [" + response + "]" );
		}

		if ( response.trim().length() == 0 )
		{
			resRC      = ReturnCodes.CODE_00048;
			resMessage = ReturnCodes.CODE_00048_MESSAGE;

			logger.error( "Error [" + resRC + " - " + resMessage + "]" );
		}
		else
		{
			/////////////////////////////////////////////////////////////////
			// test to check if data is an xml ERROR structure
			// if so, get STATUS information
			//
			ErrorHandler handlerError = new ErrorHandler();
			new SAXManager().parseSAXString( handlerError, response.trim() );

			if ( handlerError.isErrorMessage() )
			{
				response   = "";
				resRC      = ReturnCodes.CODE_00200;
				resMessage = handlerError.getStatus();

				logger.error( "Error [" + resRC + " - " + resMessage + "]" );
			}
			else // not an error message
			{
				/////////////////////////////////////////////////////////////////
				// parse ADMISSION_RESP message and get CONFIRM number
				// then build response message
				//
				AdmissionResponseHandler handler = new AdmissionResponseHandler();
				new SAXManager().parseSAXString( handler, response.trim() );

				if ( handler.isAdmissionResponse() )
				{
					response   = "<ADMISSION_RESP Type='C'>" + handler.getConfirm() + "</ADMISSION_RESP>";
					resRC      = handler.getReturnCode();
					resMessage = handler.getReturnMessage();

					this.processEmails( (Vector) handler.getEmails() );
				}
			}
		}

		this.getXMLFactsMessage().setRequestData(   response   );
		this.getXMLFactsMessage().setReturnCode(    resRC      );
		this.getXMLFactsMessage().setReturnMessage( resMessage );


		//////////////////////////////////////////////////////////////
		// build response and store
		//
		int lenCB    = this.getEDIControlBlock().SIZE;
		int lenFM    = this.getXMLFactsMessage().getXML().length();
		int lenTotal = lenCB + lenFM;

	    this.getEDIControlBlock().setTotalRecordSize( String.valueOf( (int) lenTotal ) );

		this.setResponse(   this.getEDIControlBlock().getData() +
						    this.getXMLFactsMessage().getXML() );
	}


	private void
	processEmails( Vector p_vEmails )
	{
		String mailActive = (String) this.getProperties().getProperty( "mail.notification" );

		if ( ! mailActive.equalsIgnoreCase( "ON" ) )
		{
		    //System.out.println( "Email notification is [" + mailActive + "]. (message.ADMISSION.properties)" );
			logger.warn( "Email notification is [" + mailActive + "]. (message.ADMISSION.properties)" );
		}
		// notification is ON !
		else
		{
		    if ( logger.isDebugEnabled() )
			{
				logger.debug( "Sending [" + p_vEmails.size() + "] email notifications ... " );
			}

			Properties prop     = (Properties) PropertyManager.getProperty( "mail.properties" );
			MailSender mail     = new MailSender( prop );
			Email      email    = null;
			Vector     vAddress = null;

			for ( int i = 0; i < p_vEmails.size(); i++ )
			{
				email = (Email) p_vEmails.elementAt( i );

				if ( logger.isDebugEnabled() )
				{
					logger.debug( "Sending [" + i + "] " + email.toString() );
				}

				// add FROM addresses
				vAddress = (Vector) email.getAddressFROM();
				if ( vAddress != null )
				{
					for ( int j = 0; j < vAddress.size(); j++ )
					{
						// set the intial from address
						if ( j == 0 ) { mail.setAddressFROM( (String) vAddress.elementAt( j ) ); }

						mail.addAddressFROM( (String) vAddress.elementAt( j ) );
					}
				}

				// add TO addresses
				vAddress = (Vector) email.getAddressTO();
				if ( vAddress != null )
				{
					for ( int j = 0; j < vAddress.size(); j++ )
					{
						mail.addAddressTO( (String) vAddress.elementAt( j ) );
					}
				}

				// add CC addresses
				vAddress = (Vector) email.getAddressCC();
				if ( vAddress != null )
				{
					for ( int j = 0; j < vAddress.size(); j++ )
					{
						mail.addAddressCC( (String) vAddress.elementAt( j ) );
					}
				}

				// add BCC addresses
				vAddress = (Vector) email.getAddressBCC();
				if ( vAddress != null )
				{
					for ( int j = 0; j < vAddress.size(); j++ )
					{
						mail.addAddressBCC( (String) vAddress.elementAt( j ) );
					}
				}

				try
				{
					mail.setContentType( ( String ) email.getContentType() );
					mail.setSubject( ( String ) email.getSubject() );
					mail.setMessage( ( String ) email.getBody() );
					mail.send();
					mail.clear();
				}
				catch ( MessagingException ex )
				{
					logger.error( "Sending mail notification [" + ex.getMessage() + "]" );
				}
			}
		}
	}
}

