package fccsc.manager.data.process;

import intarsys.util.*;

import fccsc.manager.broker.*;
import fccsc.manager.data.*;
import fccsc.manager.data.edi.*;


public final class CourseList
	extends ProcessFEDI
{
    public
	CourseList()
    {
		super( "message.COURSELIST.properties" );
    }


	/**
	 * Interface Method - Processing method.
	 * @throws Exception
	 */
	public void
	process() throws Exception
	{
		super.process();

		// setup our return codes
		String resRC      = ReturnCodes.CODE_00000;
		String resMessage = ReturnCodes.CODE_00000_MESSAGE;
		String clData     = "";


		String studId  = this.getEDIStandardDataBlock().getStudentID();
		String studPwd = this.getEDIStandardDataBlock().getPinPassword();

		studId  = (String) StringTools.padRight( studId,  9, ' ' );
		studPwd = (String) StringTools.padRight( studPwd, 6, ' ' );

		String data = studId + studPwd;

		EntireXBroker broker = new EntireXBroker( this.getProperties() );
		broker.sendMessage( data );

		// get response message
		String response      = (String) broker.getResponse();
		//String resParams     = (String) response.substring(   0,   0 + 15    );
		//String resExpand     = (String) response.substring(  15,  15 + 85    );
		String resCourseList = (String) response.substring( 100, 100 + 25900 );
		//System.out.println( "CourseList [" + resCourseList + "]" );


		if ( logger.isDebugEnabled() )
		{
			logger.debug( "Broker Response [" + response + "]" );
		}

		if ( resCourseList.trim().length() == 0 )
		{
			resRC      = ReturnCodes.CODE_00048;
	    	resMessage = ReturnCodes.CODE_00048_MESSAGE;
			clData     = "";

			logger.error( "Error [" + resRC + " - " + resMessage + "]" );
		}
		else
		{
			CourseListBlock ediCLB = new CourseListBlock( studId, resCourseList );
	    	ediCLB.buildResponse();
			clData = ediCLB.getData();
		}

		this.getEDIStandardDataBlock().setReturnCode(    resRC );
		this.getEDIStandardDataBlock().setReturnMessage( resMessage );


		//////////////////////////////////////////////////////////////
		// build response and store
		//
		StringBuffer buffer = new StringBuffer();

		String dataPDB = this.getRequest().substring( ControlBlock.SIZE + StandardDataBlock.SIZE );

		//int lenTRS   = Integer.parseInt( (String) this.getEDIControlBlock().getTotalRecordSize() );
		int lenPDB   = dataPDB.length();
		int lenCL    = clData.length();
		int lenTotal = ControlBlock.SIZE + StandardDataBlock.SIZE + lenPDB + lenCL;

		this.getEDIControlBlock().setTotalRecordSize( String.valueOf( (int) lenTotal ) );

		buffer.append( (String) this.getEDIControlBlock().getData() );
		buffer.append( (String) this.getEDIStandardDataBlock().getData() );
		buffer.append( (String) dataPDB );
		buffer.append( (String) clData );

		this.setResponse( (String) buffer.toString() );
	}
}